﻿/*
Dans le jeu :
'#' -> mur
' ' -> sol ou un espace vide
'.' -> trou
'$' -> boule sur un sol
'*' -> boule sur un trou
'@' -> Héros
'+' -> Héros sur un trou
*/
import mx.utils.Delegate;
class Sokoban {
	private var scene:MovieClip;
	private var mon_fmt:TextFormat;
	private var pseudo:String;
	private var plateau:MovieClip;
	private var mesNiveaux:XML;
	private var niveau:Number;
	private var niveaux:String;
	private var nbBoules:Number;
	private var nbBoulesMises:Number;
	private var nbCoups:Number;
	private var nbCoupsTot:Number;
	private var nomNiveau:String;
	private var tabNiveau:Array;
	private var tabBoules:Array;
	private var posHerosX:Number;
	private var posHerosY:Number;
	private var objAnnule:Object;
	private var enregistrement:Number;
	private var monEnvoi:LoadVars;
	// Déclaration de mes variables de classe pour paramétrer mon jeu
	private static var DEBUTX:Number = 20;
	private static var DEBUTY:Number = 50;
	private static var TXTX:Number = 5;
	private static var TXTY:Number = -45;
	private static var TXTLARG:Number = 450;
	private static var TXTHAUT:Number = 40;
	private static var TAILLE:Number = 20;
	private static var PROFHEROS:Number = 5000;
	private static var NOMSHARE:String = "sokonyro";
	private static var CLFOND:String = "000000";
	public function Sokoban(scen:MovieClip, niveau:String) {
		scene = scen;
		niveaux = niveau;
		// Création du fond
		scene.createEmptyMovieClip("fond_mc", -10);
		scene.fond_mc._x = 0;
		scene.fond_mc._y = 0;
		scene.fond_mc.moveTo(0, 0);
		scene.fond_mc.lineStyle(0, "0x"+CLFOND, 100);
		scene.fond_mc.beginFill("0x"+CLFOND, 100);
		scene.fond_mc.lineTo(550, 0);
		scene.fond_mc.lineTo(550, 400);
		scene.fond_mc.lineTo(0, 400);
		scene.fond_mc.endFill();
		scene.createEmptyMovieClip("debut", -9);
		// Création du champ texte d'info
		scene.debut.createTextField("info_txt", 1, 70, 70, 410, 45);
		scene.debut.info_txt.text = "Entrez votre Pseudo :";
		scene.debut.info_txt.selectable = false;
		mon_fmt = new TextFormat();
		mon_fmt.font = "Verdana";
		mon_fmt.size = 35;
		mon_fmt.color = 0xFFCC33;
		mon_fmt.align = "center";
		scene.debut.info_txt.setTextFormat(mon_fmt);
		// Création du champ texte pour le pseudo
		scene.debut.createTextField("pseudo_txt", 2, 70, 140, 410, 45);
		scene.debut.pseudo_txt.type = "input";
		scene.debut.pseudo_txt.background = 0xDDDDDD;
		changePseud();
		scene.debut.pseudo_txt.onChanged = Delegate.create(this, changePseud);
		var initObj:Object = new Object();
		initObj._x = 350;
		initObj._y = 230;
		initObj.onRelease = Delegate.create(this, pressBouton);
		scene.debut.attachMovie("bmc", "bouton_btn", -7, initObj);
	}
	private function changePseud(Void):Void {
		scene.debut.pseudo_txt.setTextFormat(mon_fmt);
	}
	private function pressBouton(Void):Void {
		if (scene.debut.pseudo_txt.text.length>0) {
			pseudo = scene.debut.pseudo_txt.text;
			// On efface les éléments
			scene.debut._visible = false;
			// On lance le jeu
			jeu();
		}
	}
	private function jeu(Void):Void {
		mesNiveaux = new XML();
		mesNiveaux.ignoreWhite = true;
		niveau = 1;
		// Ici on dit que notre onload sera la fonction chargementNiveaux
		//		de note objet Jeu, via Delegate.create
		mesNiveaux.onLoad = Delegate.create(this, chargementNiveaux);
		mesNiveaux.load(niveaux);
		creerPlateau();
		nbCoupsTot = 0;
		var monEcouteur:Object = monEcouteur=new Object();
		Key.addListener(monEcouteur);
		// Ici on dit que la fonction bouge (appelé lors de la pression d'une touche)
		//		sera bouge, via Delegate.create
		monEcouteur.onKeyDown = Delegate.create(this, bouge);
		// On place monEcouteur sur la scene pour le détruire après
		scene.monEcouteur = monEcouteur;
		// Création de la fonction de destruction
		scene.detruire = function(Void) {
			Key.removeListener(monEcouteur);
		};
	}
	private function chargementNiveaux(succes:Boolean):Void {
		if (succes) {
			constructionJeu();
		} else {
			trace("Chargement échoué");
		}
	}
	private function constructionJeu(Void):Void {
		// Initialisation des variable du niveau
		nbBoules = 0;
		nbBoulesMises = 0;
		nbCoups = 0;
		enregistrement = 1;
		objAnnule = undefined;
		// Ici, on met dans un tableau toutes les données du jeu
		//		Et on affiche nos éléments sur la scène
		var monNiveau:XMLNode = mesNiveaux.firstChild.childNodes[niveau-1];
		nomNiveau = monNiveau.attributes.Id;
		var hauteur:Number = monNiveau.attributes.Height;
		var largeur:Number = monNiveau.attributes.Width;
		tabNiveau = new Array(hauteur);
		tabBoules = new Array(hauteur);
		var ligne:String;
		var cellule:String;
		var ident:Number;
		var initObj:Object = new Object();
		var debut:Boolean;
		var aff:String;
		for (var i = 0; i<hauteur; i++) {
			tabNiveau[i] = new Array(largeur);
			tabBoules[i] = new Array(largeur);
			ligne = monNiveau.childNodes[i].firstChild.nodeValue;
			debut = false;
			for (var j = 0; j<largeur; j++) {
				cellule = ligne.charAt(j);
				tabNiveau[i][j] = cellule;
				aff = "";
				initObj._x = TAILLE*j;
				initObj._y = TAILLE*i;
				if (cellule == "#") {
					// mur
					aff = "mur";
					debut = true;
				} else if (cellule == " " && debut) {
					// sol
					aff = "sol";
				} else if (cellule == "$") {
					// Boule
					nbBoules++;
					aff = "sol";
					plateau.attachMovie("boule", "boule"+nbBoules, PROFHEROS+nbBoules, initObj);
					tabBoules[i][j] = eval(plateau+".boule"+nbBoules);
				} else if (cellule == "@") {
					// Heros
					aff = "sol";
					plateau.attachMovie("heros", "heros_mc", PROFHEROS, initObj);
					posHerosX = j;
					posHerosY = i;
				} else if (cellule == ".") {
					// trou
					aff = "trou";
				} else if (cellule == "*") {
					// Boule placée
					nbBoules++;
					aff = "trou";
					plateau.attachMovie("boule", "boule"+nbBoules, PROFHEROS+nbBoules, initObj);
					tabBoules[i][j] = plateau.eval("boule"+nbBoules);
					eval("boule"+nbBoules).gotoAndStop(2);
				} else if (cellule == "+") {
					// Héros sur trou
					aff = "trou";
					plateau.attachMovie("heros", "heros_mc", PROFHEROS, initObj);
					posHerosX = j;
					posHerosY = i;
				}
				if (aff != "") {
					ident = (100*i+j);
					plateau.attachMovie(aff, "obj"+ident, ident, initObj);
				}
			}
		}
		affichage();
	}
	private function affichage(Void):Void {
		var affiche:String = nomNiveau+" ("+niveau+") - Boules : "+nbBoulesMises+"/"+nbBoules;
		affiche += "\nCoups : "+nbCoups+" - Coups Totals : "+nbCoupsTot;
		plateau.affichage_txt.text = affiche;
	}
	private function creerPlateau(Void):Void {
		var tempx:Number = plateau._x;
		var tempy:Number = plateau._y;
		if (tempx == undefined) {
			tempx = DEBUTX;
		}
		if (tempy == undefined) {
			tempy = DEBUTY;
		}
		scene.createEmptyMovieClip("plateau", 1);
		plateau = eval(scene+".plateau");
		plateau._x = tempx;
		plateau._y = tempy;
		plateau.onPress = function() {
			startDrag(this);
		};
		plateau.onRelease = function() {
			stopDrag();
		};
		plateau.createTextField("affichage_txt", PROFHEROS-1, TXTX, TXTY, TXTLARG, TXTHAUT);
		plateau.affichage_txt.textColor = "0xFFFFFF";
	}
	private function bouge(Void):Void {
		var touche:Number = Key.getCode();
		var bgX:Number = 0;
		var bgY:Number = 0;
		if (touche == 69) {
			// touche E
			enregistre();
		} else if (touche == 83) {
			// touche S
			sauvegarde();
		} else if (touche == 82) {
			// touche r
			recov();
		} else if (touche == 32) {
			// Barre d'espace
			annule();
		} else if (touche == 37) {
			// Gauche
			bgX = -1;
		} else if (touche == 38) {
			// Haut
			bgY = -1;
		} else if (touche == 39) {
			// Droite
			bgX = 1;
		} else if (touche == 40) {
			// Bas
			bgY = 1;
		}
		if (bgX != 0 || bgY != 0) {
			var dir:String = tabNiveau[posHerosY+bgY][posHerosX+bgX];
			if (dir == " " || dir == ".") {
				if (dir == ".") {
					// On va vers un trou
					tabNiveau[posHerosY+bgY][posHerosX+bgX] = "+";
				} else {
					// On va vers un endroit vide
					tabNiveau[posHerosY+bgY][posHerosX+bgX] = "@";
				}
				if (tabNiveau[posHerosY][posHerosX] == "+") {
					tabNiveau[posHerosY][posHerosX] = ".";
				} else {
					tabNiveau[posHerosY][posHerosX] = " ";
				}
				posHerosX = posHerosX+bgX;
				posHerosY = posHerosY+bgY;
				plateau.heros_mc._x = TAILLE*posHerosX;
				plateau.heros_mc._y = TAILLE*posHerosY;
				nbCoups++;
				nbCoupsTot++;
			} else if (dir == "$" || dir == "*") {
				// On essaie de pousser une boule
				var bgX2:Number = bgX*2;
				var bgY2:Number = bgY*2;
				var dir2:String = tabNiveau[posHerosY+bgY2][posHerosX+bgX2];
				if (dir2 == " " || dir2 == ".") {
					// La boule peut-être poussée !
					enregAnnule(bgX, bgY, bgX2, bgY2);
					var boule:MovieClip = tabBoules[posHerosY+bgY][posHerosX+bgX];
					tabBoules[posHerosY+bgY2][posHerosX+bgX2] = boule;
					tabBoules[posHerosY+bgY][posHerosX+bgX] = "";
					boule._x = TAILLE*(posHerosX+bgX2);
					boule._y = TAILLE*(posHerosY+bgY2);
					if (dir2 == ".") {
						// La boule va vers un trou
						tabNiveau[posHerosY+bgY2][posHerosX+bgX2] = "*";
						boule.gotoAndStop(2);
						if (dir == "$") {
							// On va vers un endroit vide
							tabNiveau[posHerosY+bgY][posHerosX+bgX] = "@";
							// Et donc, une boule de plus au compteur !
							nbBoulesMises++;
						} else {
							// On va vers un trou
							tabNiveau[posHerosY+bgY][posHerosX+bgX] = "+";
						}
					} else {
						// La boule va vers un endroit vide
						tabNiveau[posHerosY+bgY2][posHerosX+bgX2] = "$";
						boule.gotoAndStop(1);
						if (dir == "$") {
							// On va vers un endroit vide
							tabNiveau[posHerosY+bgY][posHerosX+bgX] = "@";
						} else {
							// On va vers un trou
							tabNiveau[posHerosY+bgY][posHerosX+bgX] = "+";
							// Et donc, une boule de moins au compteur !
							nbBoulesMises--;
						}
					}
					if (tabNiveau[posHerosY][posHerosX] == "+") {
						tabNiveau[posHerosY][posHerosX] = ".";
					} else {
						tabNiveau[posHerosY][posHerosX] = " ";
					}
					posHerosX = posHerosX+bgX;
					posHerosY = posHerosY+bgY;
					plateau.heros_mc._x = TAILLE*posHerosX;
					plateau.heros_mc._y = TAILLE*posHerosY;
					nbCoups++;
					nbCoupsTot++;
				}
			}
		}
		if (nbBoulesMises == nbBoules) {
			niveau++;
			creerPlateau();
			constructionJeu();
		}
		affichage();
	}
	private function enregAnnule(bouleAvtX:Number, bouleAvtY:Number, bouleAprX:Number, bouleAprY:Number):Void {
		objAnnule = new Object();
		objAnnule.posHerosX = posHerosX;
		objAnnule.posHerosY = posHerosY;
		objAnnule.bouleAvtX = posHerosX+bouleAvtX;
		objAnnule.bouleAvtY = posHerosY+bouleAvtY;
		objAnnule.bouleAprX = posHerosX+bouleAprX;
		objAnnule.bouleAprY = posHerosY+bouleAprY;
	}
	private function annule(Void):Void {
		if (objAnnule != undefined) {
			// Remise en place du héros
			if (tabNiveau[posHerosY][posHerosX] == "+") {
				// Le héros est sur un trou
				tabNiveau[posHerosY][posHerosX] = ".";
			} else {
				tabNiveau[posHerosY][posHerosX] = " ";
			}
			posHerosX = objAnnule.posHerosX;
			posHerosY = objAnnule.posHerosY;
			if (tabNiveau[posHerosY][posHerosX] == ".") {
				// Le héros est sur un trou
				tabNiveau[posHerosY][posHerosX] = "+";
			} else {
				tabNiveau[posHerosY][posHerosX] = "@";
			}
			plateau.heros_mc._x = TAILLE*posHerosX;
			plateau.heros_mc._y = TAILLE*posHerosY;
			// Remise de la boule
			var boule:MovieClip = tabBoules[objAnnule.bouleAprY][objAnnule.bouleAprX];
			boule._x = TAILLE*objAnnule.bouleAvtX;
			boule._y = TAILLE*objAnnule.bouleAvtY;
			if (tabNiveau[objAnnule.bouleAprY][objAnnule.bouleAprX] == "*") {
				// La boule est sur un trou
				tabNiveau[objAnnule.bouleAprY][objAnnule.bouleAprX] = ".";
				nbBoules--;
			} else {
				tabNiveau[objAnnule.bouleAprY][objAnnule.bouleAprX] = " ";
			}
			if (tabNiveau[objAnnule.bouleAvtY][objAnnule.bouleAvtX] == ".") {
				// La boule était sur un trou
				tabNiveau[objAnnule.bouleAvtY][objAnnule.bouleAvtX] = "*";
				nbBoules++;
				boule.gotoAndStop(2);
			} else {
				tabNiveau[objAnnule.bouleAvtY][objAnnule.bouleAvtX] = "$";
				boule.gotoAndStop(1);
			}
			tabBoules[objAnnule.bouleAprY][objAnnule.bouleAprX] = "";
			tabBoules[objAnnule.bouleAvtY][objAnnule.bouleAvtX] = boule;
			nbCoups++;
			nbCoupsTot++;
		}
	}
	private function sauvegarde(Void):Void {
		if (enregistrement>0) {
			var monShare:SharedObject = new SharedObject();
			monShare = SharedObject.getLocal(NOMSHARE);
			monShare.data.jeu = this;
			monShare.flush();
			enregistrement--;
		}
	}
	private function recov(Void):Void {
		var monShare:SharedObject = new SharedObject();
		monShare = SharedObject.getLocal(NOMSHARE);
		var sauv:Object = monShare.data.jeu;
		if (sauv != null) {
			pseudo = sauv.pseudo;
			mesNiveaux = sauv.mesNiveaux;
			niveau = sauv.niveau;
			// Remise des variables
			nbBoules = 0;
			nbBoulesMises = 0;
			nbCoups = sauv.nbCoups;
			nbCoupsTot = sauv.nbCoupsTot;
			nomNiveau = sauv.nomNiveau;
			tabNiveau = sauv.tabNiveau;
			tabBoules = sauv.tabBoules;
			posHerosX = sauv.posHerosX;
			posHerosY = sauv.posHerosY;
			objAnnule = sauv.objAnnule;
			enregistrement = sauv.enregistrement;
			var monNiveau:XMLNode = mesNiveaux.firstChild.childNodes[niveau-1];
			nomNiveau = monNiveau.attributes.Id;
			var hauteur:Number = tabNiveau.length;
			var largeur:Number = tabNiveau[0].length;
			var ident:Number;
			var initObj:Object = new Object();
			var debut:Boolean;
			var cellule:String;
			var aff:String;
			creerPlateau();
			for (var i = 0; i<hauteur; i++) {
				debut = false;
				for (var j = 0; j<largeur; j++) {
					cellule = tabNiveau[i][j];
					aff = "";
					initObj._x = TAILLE*j;
					initObj._y = TAILLE*i;
					if (cellule == "#") {
						// mur
						aff = "mur";
						debut = true;
					} else if (cellule == " " && debut) {
						// sol
						aff = "sol";
					} else if (cellule == "$") {
						// Boule
						aff = "sol";
						nbBoules++;
						plateau.attachMovie("boule", "boule"+nbBoules, PROFHEROS+nbBoules, initObj);
						tabBoules[i][j] = eval(plateau+".boule"+nbBoules);
						eval(plateau+".boule"+nbBoules).gotoAndStop(1);
					} else if (cellule == "@") {
						// Heros
						aff = "sol";
						plateau.attachMovie("heros", "heros_mc", PROFHEROS, initObj);
						posHerosX = j;
						posHerosY = i;
					} else if (cellule == ".") {
						// trou
						aff = "trou";
					} else if (cellule == "*") {
						// Boule placée
						aff = "trou";
						nbBoules++;
						plateau.attachMovie("boule", "boule"+nbBoules, PROFHEROS+nbBoules, initObj);
						tabBoules[i][j] = eval(plateau+".boule"+nbBoules);
						eval(plateau+".boule"+nbBoules).gotoAndStop(2);
						nbBoulesMises++;
					} else if (cellule == "+") {
						// Héros sur trou
						aff = "trou";
						plateau.attachMovie("heros", "heros_mc", PROFHEROS, initObj);
						posHerosX = j;
						posHerosY = i;
					}
					if (aff != "") {
						ident = (100*i+j);
						plateau.attachMovie(aff, "obj"+ident, ident, initObj);
					}
				}
			}
		}
	}
	private function enregistre(Void):Void {
		if (niveau>1) {
			monEnvoi = new LoadVars();
			monEnvoi.pseudo = pseudo;
			monEnvoi.niveau = niveau;
			monEnvoi.nbCoupsTot = nbCoupsTot;
			monEnvoi.sendAndLoad("enregsoko.php", monEnvoi, "POST");
			// Ici on dit que notre onload sera la fonction enregistreAction
			//		de note objet Jeu, via Delegate.create
			monEnvoi.onLoad = Delegate.create(this, enregistreAction);
		}
	}
	private function enregistreAction(succes:Boolean):Void {
		if (succes && monEnvoi.res == 1) {
			var monShare:SharedObject = new SharedObject();
			monShare = SharedObject.getLocal(NOMSHARE);
			monShare.data.jeu = null;
			creerPlateau();
			nbCoupsTot = 0;
			niveau = 1;
			constructionJeu();
		} else {
			trace("Erreur pendant l'enregistrement du score");
		}
	}
}
